package usecase

import (
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/entity"
	repo "code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/repository"
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/usecase"
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/repository"
	"code.30cm.net/digimon/library-go/errs"
	"context"
	"github.com/zeromicro/go-zero/core/logx"
)

type TagsUseCaseParam struct {
	TagsRepo repository.TagsRepository
}

type TagsUseCase struct {
	TagsUseCaseParam
}

func MustTagsUseCase(param TagsUseCaseParam) usecase.ProductBaseTags {
	return &TagsUseCase{
		param,
	}
}

func (use *TagsUseCase) Create(ctx context.Context, data *entity.Tags) error {
	err := use.TagsRepo.Create(ctx, data)
	if err != nil {
		return errs.DBErrorL(logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "param", Value: data},
				{Key: "func", Value: "TagsRepo.Create"},
				{Key: "err", Value: err.Error()},
			}, "failed to create tags")
	}

	return nil
}

func (use *TagsUseCase) GetByID(ctx context.Context, id string) (*entity.Tags, error) {
	tag, err := use.TagsRepo.GetByID(ctx, id)
	if err != nil {
		return nil, errs.DBErrorL(logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "id", Value: id},
				{Key: "func", Value: "TagsRepo.GetByID"},
				{Key: "err", Value: err.Error()},
			}, "failed to get tags")
	}

	return tag, nil
}

func (use *TagsUseCase) GetByIDs(ctx context.Context, ids []string) ([]*entity.Tags, error) {
	tags, err := use.TagsRepo.GetByIDs(ctx, ids)
	if err != nil {
		return nil, errs.DBErrorL(logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "ids", Value: ids},
				{Key: "func", Value: "TagsRepo.GetByIDs"},
				{Key: "err", Value: err.Error()},
			}, "failed to get tags")
	}

	return tags, nil
}

func (use *TagsUseCase) Delete(ctx context.Context, id string) error {
	// TODO 是否要刪除已綁定的Tag 還是在查詢的時候做

	err := use.TagsRepo.Delete(ctx, id)
	if err != nil {
		return errs.DBErrorL(logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "id", Value: id},
				{Key: "func", Value: "TagsRepo.Delete"},
				{Key: "err", Value: err.Error()},
			}, "failed to delete tags")
	}

	return nil
}

func (use *TagsUseCase) List(ctx context.Context, params usecase.TagQueryParams) ([]*entity.Tags, int64, error) {
	q := repo.TagQueryParams{
		PageSize:  params.PageSize,
		PageIndex: params.PageIndex,
	}
	if params.Types != nil {
		q.Types = params.Types
	}

	if params.Name != nil {
		q.Name = params.Name
	}
	if params.ShowType != nil {
		q.ShowType = params.ShowType
	}

	list, total, err := use.TagsRepo.List(ctx, q)
	if err != nil {
		return nil, 0, errs.DBErrorL(logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "params", Value: params},
				{Key: "func", Value: "TagsRepo.List"},
				{Key: "err", Value: err.Error()},
			}, "failed to list tags")
	}

	return list, total, nil
}

func (use *TagsUseCase) Update(ctx context.Context, id string, tag usecase.TagModifyParams) error {
	update := repo.TagModifyParams{}
	if tag.Name != nil {
		update.Name = tag.Name
	}
	if tag.Types != nil {
		update.Types = tag.Types
	}
	if tag.ShowType != nil {
		update.ShowType = tag.ShowType
	}
	if tag.Cover != nil {
		update.Cover = tag.Cover
	}

	err := use.TagsRepo.Update(ctx, id, update)
	if err != nil {
		return errs.DBErrorL(logx.WithContext(ctx),
			[]logx.LogField{
				{Key: "id", Value: id},
				{Key: "params", Value: tag},
				{Key: "func", Value: "TagsRepo.Update"},
				{Key: "err", Value: err.Error()},
			}, "failed to update tags")
	}

	return nil
}
